'use strict';

var _chai = require('chai');

var _nock = require('nock');

var _nock2 = _interopRequireDefault(_nock);

var _dist = require('../dist');

var _dist2 = _interopRequireDefault(_dist);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint no-unused-expressions: 0 */
describe('actions', function () {
  it('cancel()', function (done) {
    var isSuccess = false;
    var isError = false;
    (0, _nock2.default)(_utils.ROOT).get('/delay').delayConnection(200).reply(200, {
      status: 0
    });
    var req = (0, _dist2.default)({
      url: _utils.ROOT + '/delay',
      success: function success() {
        isSuccess = true;
      },
      error: function error() {
        isError = true;
      }
    });
    var startTime = new Date().getTime();
    function fetchDone() {
      (0, _chai.expect)(isSuccess).to.be.false;
      (0, _chai.expect)(isError).to.be.false;
      (0, _chai.expect)(new Date().getTime() - startTime).to.be.above(300);
      done();
    }
    req.send().then(fetchDone).catch(fetchDone);
    setTimeout(function () {
      req.cancel();
    }, 100);
    setTimeout(fetchDone, 300);
  });

  it('error()', function (done) {
    var isSuccess = false;
    var isError = false;
    (0, _nock2.default)(_utils.ROOT).get('/delay').delayConnection(200).reply(200, {
      status: 0
    });
    var req = (0, _dist2.default)({
      url: _utils.ROOT + '/delay',
      success: function success() {
        isSuccess = true;
      },
      error: function error(res) {
        isError = true;
        (0, _chai.expect)(res.status).to.equal(6);
        (0, _chai.expect)(res.customError).to.be.an('error');
        (0, _chai.expect)(res.customError.toString()).to.be.include('test');
      }
    });
    var startTime = new Date().getTime();
    function fetchDone() {
      (0, _chai.expect)(isSuccess).to.be.false;
      (0, _chai.expect)(isError).to.be.true;
      (0, _chai.expect)(new Date().getTime() - startTime).to.be.above(300);
      done();
    }
    req.send().then(fetchDone).catch(fetchDone);
    setTimeout(function () {
      req.error(new Error('test'));
    }, 100);
    setTimeout(fetchDone, 300);
  });
});
//# sourceMappingURL=actions.spec.js.map