'use strict';

var _chai = require('chai');

var _nock = require('nock');

var _nock2 = _interopRequireDefault(_nock);

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

var _dist = require('../dist');

var _dist2 = _interopRequireDefault(_dist);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('put', function () {
  it('user3, by urlencoded', function (done) {
    (0, _nock2.default)(_utils.ROOT, {
      reqheaders: {
        'X-My-Headers': '1',
        'Content-Type': /application\/x-www-form-urlencoded/
      }
    }).defaultReplyHeaders({
      'X-Powered-By': 'nock'
    }).put('/user').query({
      uid: 3
    }).reply(201, function (uri, requestBody) {
      return Object.assign(_qs2.default.parse(requestBody), {
        status: 0
      });
    });
    (0, _dist2.default)({
      url: _utils.ROOT + '/user',
      method: 'put',
      query: {
        uid: 3
      },
      headers: {
        'X-My-Headers': '1'
      },
      data: {
        name: 'user3'
      },
      mergeHeaders: {
        poweredBy: 'X-Powered-By'
      }
    }).send().then(function (res) {
      (0, _chai.expect)(res.name).to.equal('user3');
      (0, _chai.expect)(res.status).to.equal(0);
      (0, _chai.expect)(res.poweredBy).to.equal('nock');
      done();
    }).catch(function (err) {
      console.log(err);
    });
  });

  it('user4, by JSON', function (done) {
    (0, _nock2.default)(_utils.ROOT, {
      reqheaders: {
        'X-My-Headers': '1',
        'Content-Type': /json/
      }
    }).put('/user').query({
      uid: 4
    }).reply(201, function (uri, requestBody) {
      return Object.assign(requestBody, {
        status: 0
      });
    });
    (0, _dist2.default)({
      url: _utils.ROOT + '/user',
      method: 'put',
      query: {
        uid: 4
      },
      headers: {
        'X-My-Headers': '1'
      },
      dataType: 'json',
      data: {
        name: 'user4'
      }
    }).send().then(function (res) {
      (0, _chai.expect)(res.name).to.equal('user4');
      (0, _chai.expect)(res.status).to.equal(0);
      done();
    }).catch(function (err) {
      console.log(err);
    });
  });
}); /* eslint no-unused-expressions: 0 */
//# sourceMappingURL=put.spec.js.map